# coding: utf-8
"""

常用技术分析指标：MA, MACD, BOLL
"""


def ma(kline, params=(5, 10, 20, 60, 120, 250)):
    """计算指定周期的若干 MA 均线

    :param kline: pd.DataFrame
        K线，columns = ["symbol", "dt", "open", "close", "high", "low", "vol"]
    :param params: tuple
    :return: pd.DataFrame
        在原始数据中新增若干 MA 均线
    """
    for p in params:
        col = "ma" + str(p)
        kline.loc[:, col] = kline['close'].rolling(p).mean().apply(round, args=(2,))
    return kline


def macd(kline):
    """计算 MACD 指标

    :param kline: pd.DataFrame
        K线，columns = ["symbol", "dt", "open", "close", "high", "low", "vol"]
    :return: pd.DataFrame
        在原始数据中新增 diff,dea,macd 三列
    """

    short_, long_, m = 12, 26, 9
    kline.loc[:, 'diff'] = kline['close'].ewm(adjust=False, alpha=2 / (short_ + 1), ignore_na=True).mean() - \
                            kline['close'].ewm(adjust=False, alpha=2 / (long_ + 1), ignore_na=True).mean()
    kline.loc[:, 'dea'] = kline['diff'].ewm(adjust=False, alpha=2 / (m + 1), ignore_na=True).mean()
    kline.loc[:, 'macd'] = 2 * (kline['diff'] - kline['dea'])

    for col in ['diff', 'dea', 'macd']:
        kline.loc[:, col] = kline[col].apply(round, args=(2,))
    return kline


def boll(kline):
    """计算 BOLL 指标

    :param kline: pd.DataFrame
        K线，columns = ["symbol", "dt", "open", "close", "high", "low", "vol"]
    :return: pd.DataFrame
        在原始数据中新增 BOLL 指标结果
    """
    kline.loc[:, 'boll-mid'] = kline['close'].rolling(26).mean()
    kline.loc[:, 'boll-tmp2'] = kline['close'].rolling(20).std()
    kline.loc[:, 'boll-top'] = kline['boll-mid'] + 2 * kline['boll-tmp2']
    kline.loc[:, 'boll-bottom'] = kline['boll-mid'] - 2 * kline['boll-tmp2']

    for col in ['boll-mid', 'boll-top', 'boll-bottom']:
        kline.loc[:, col] = kline[col].apply(round, args=(2,))
    return kline



# coding: utf-8

from .analyze import KlineAnalyze, is_bei_chi
from .ta import ma, macd, boll
from .utils import plot_kline
from .solid import SolidAnalyze
from .solid import is_in_tolerance, is_first_buy, is_first_sell, is_second_buy, \
    is_second_sell, is_third_buy, is_third_sell, is_xd_buy, is_xd_sell

__version__ = "0.3.8"
__author__ = "zengbin93"
__email__ = "zeng_bin8888@163.com"





# coding: utf-8
import sys
from cobra.data.kline import get_kline
sys.path.insert(0, '.')
sys.path.insert(0, '..')
import czsc
from czsc import KlineAnalyze
from czsc.analyze import is_bei_chi, find_zs
from czsc.utils import plot_ka

print(czsc.__version__)

df = get_kline(ts_code="000001.SH", end_dt="2020-04-28 15:00:00", freq='D', asset='I')
ka = KlineAnalyze(df, name="日线")
print(ka)


def test_kline_analyze():
    assert ka.bi[-1]['fx_mark'] == 'g'
    assert ka.xd[-1]['fx_mark'] == 'd'

    # 测试背驰识别
    assert not ka.bi_bei_chi()
    assert ka.xd_bei_chi()
    print(ka.zs[-2])

    ka.to_html("kline.html")
    ka.to_image("kline.png")


def test_bei_chi():
    # 线段背驰
    zs1 = {"start_dt": '2018-07-26 15:00:00', "end_dt": '2018-10-19 15:00:00', "direction": "down"}
    zs2 = {"start_dt": '2018-01-29 15:00:00', "end_dt": '2018-07-06 15:00:00', "direction": "down"}
    assert is_bei_chi(ka, zs1, zs2, mode='xd', adjust=0.9)

    zs1 = {"start_dt": '2013-12-10 15:00:00', "end_dt": '2014-01-20 15:00:00', "direction": "down"}
    zs2 = {"start_dt": '2013-09-12 15:00:00', "end_dt": '2013-11-14 15:00:00', "direction": "down"}
    assert not is_bei_chi(ka, zs1, zs2, mode='xd', adjust=0.9)

    # 笔背驰
    zs1 = {"start_dt": '2019-05-17 15:00:00', "end_dt": '2019-06-10 15:00:00'}
    zs2 = {"start_dt": '2019-04-08 15:00:00', "end_dt": '2019-05-10 15:00:00'}
    assert is_bei_chi(ka, zs1, zs2, mode='bi', adjust=0.9)

    zs1 = {"start_dt": '2018-09-28 15:00:00', "end_dt": '2018-10-19 15:00:00'}
    zs2 = {"start_dt": '2018-08-28 15:00:00', "end_dt": '2018-09-12 15:00:00'}
    assert not is_bei_chi(ka, zs1, zs2, mode='bi', adjust=0.9)


def test_find_zs():
    assert ka.down_zs_number() == 2
    assert ka.up_zs_number() == 1
    xd_zs = find_zs(ka.xd)
    bi_zs = find_zs(ka.bi)

    assert xd_zs[-2]["ZD"] == 2850.71
    assert xd_zs[-2]["ZG"] == 3684.57

    assert xd_zs[-1]["ZD"] == 2691.02
    assert xd_zs[-1]["ZG"] == 2827.34

    assert bi_zs[-2]['ZD'] == 2987.77
    assert bi_zs[-2]['ZG'] == 3125.02

    assert bi_zs[-1]['ZD'] == 2838.38
    assert bi_zs[-1]['ZG'] == 2956.78

